"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const child_process_1 = require("child_process");
const fs_1 = require("fs");
const path_1 = require("path");
const process_1 = require("process");
function notBlank(s) {
    return s != null && String(s).trim().length > 0;
}
function findPackageVersion(dir) {
    const path = path_1.resolve(path_1.join(dir, "package.json"));
    try {
        const json = JSON.parse(fs_1.readFileSync(path).toString());
        if (json != null) {
            if (notBlank(json.version)) {
                return { version: json.version, dir };
            }
            else {
                throw new Error("No `version` field was found in " + path);
            }
        }
    }
    catch (err) {
        const parent = path_1.resolve(path_1.join(dir, ".."));
        if (path_1.resolve(dir) !== parent) {
            return findPackageVersion(parent);
        }
        else {
            throw err;
        }
    }
}
function headSha(cwd) {
    const gitSha = child_process_1.execSync("git rev-parse -q HEAD", { cwd }).toString().trim();
    if (gitSha.length < 40) {
        throw new Error("Unexpected git SHA: " + gitSha);
    }
    else {
        return gitSha;
    }
}
function headUnixtime(cwd) {
    const unixtimeStr = child_process_1.execSync("git log -1 --pretty=format:%ct", {
        cwd,
    }).toString();
    const unixtime = parseInt(unixtimeStr);
    const date = new Date(unixtime * 1000);
    if (date > new Date() || date < new Date(2000, 0, 1)) {
        throw new Error("Unexpected unixtime for commit: " + unixtime);
    }
    return date;
}
function ymdhms(d) {
    // NOTE: I thought about only including the hour, or the hour and minute, but
    // this value is still < Javascript's max integer (2^53), so the whole thing
    // is fine (and probably a bit more defensible to the Engineers of Tomorrow).
    // Math.log2(20180919202444) = 44.1
    return d
        .toISOString()
        .replace(/[^0-9]/g, "")
        .substring(0, 14);
}
exports.ymdhms = ymdhms;
function renderVersionInfo(o) {
    const msg = [];
    const ts = o.output.endsWith(".ts");
    if (!ts) {
        msg.push(`"use strict";`, `Object.defineProperty(exports, "__esModule", { value: true });`);
    }
    for (const ea of [
        `version = "${o.version}"`,
        `release = "${o.release}"`,
        `gitSha = "${o.gitSha}"`,
        `gitDate = new Date(${o.gitDate.getTime()})`,
    ]) {
        msg.push(ts ? `export const ${ea};` : `exports.${ea};`);
    }
    return msg.join("\n") + "\n";
}
function mkver(output = path_1.join(process_1.cwd(), "Version.ts")) {
    const file = path_1.resolve(path_1.normalize(output));
    const parsed = path_1.parse(file);
    try {
        const v = findPackageVersion(parsed.dir);
        if (v == null) {
            throw new Error("No package.json was found in " + parsed.dir + " or parent directories.");
        }
        const gitSha = headSha(v.dir);
        const gitDate = headUnixtime(v.dir);
        const msg = renderVersionInfo({
            output,
            version: v.version,
            release: `${v.version}+${ymdhms(gitDate)}`,
            gitSha,
            gitDate,
        });
        try {
            fs_1.mkdirSync(parsed.dir, { recursive: true });
        }
        catch (err) {
            if (err.code !== "EEXIST")
                throw err;
        }
        fs_1.writeFileSync(file, msg);
    }
    catch (err) {
        throw new Error(process_1.argv[1] + ": Failed to produce " + output + ":\n  " + err.message);
    }
}
exports.mkver = mkver;
//# sourceMappingURL=mkver.js.map